/*
	*
	* This file is a part of IonShell.
	* IonShell is the Desktop Shell App for Ion DE
	* Copyright 2020 CuboCore Group
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* ( at your option ) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#include <QDebug>
#include <QWindow>

#include "qwayland-wayfire-shell-unstable-v2.h"

#include "LayerShell.hpp"
#include "DesQWlGlobal.hpp"
#include "AutoHideWindow.hpp"
#include "AutoHideWindowImpl.hpp"

// TODO : A slide down animation to hide the dock

DesQAutoHideWindow::DesQAutoHideWindow( QWindow *window, zwf_shell_manager_v2 *wayfireShell, QObject *parent ) : QObject( parent ) {

	m_window = window;
	m_wayfireShell = wayfireShell;

	m_dockPos = 2; // Currently we are creating a bottom dockbar
	m_edgeHeight = 20;
	m_showDuration = 150;
	m_hideDuration = 2000;

	m_isEdgeHotspot = false; // False meant DesQHotSpot is there
	m_hideTimer = new QTimer;
	m_hideTimer->setInterval( m_hideDuration );

	/* Two hotspot need for a autohiding window
	   One to show the window ( edgehotspot ) and
	   another to hide the window ( dockHotSpot ) */
	m_edgeHotspot = new DesQHotSpot( m_window->screen(), m_wayfireShell, this );
	m_dockHotspot = new DesQHotSpot( m_window->screen(), m_wayfireShell, this );

	connect(
		m_edgeHotspot, &DesQHotSpot::enteredHotspot, [=]() {
			qDebug() << "Edge hotspot entered";
			m_isEdgeHotspot = true;
			updateMargin();
			m_hideTimer->stop();
		}
	);

	connect(
		m_edgeHotspot, &DesQHotSpot::leftHotspot, [=]() {
			qDebug() << "Edge hotspot left";
			createDockHS();
		}
	);

	connect(
		m_dockHotspot, &DesQHotSpot::enteredHotspot, [=]() {
			qDebug() << "Panel Hotspot entered";
			// Set any value to the timer
			m_hideTimer->stop();
		}
	);

	connect(
		m_dockHotspot, &DesQHotSpot::leftHotspot, [=]() {
			qDebug() << "Panel Hotspot left";
			m_hideTimer->start();
		}
	);

	connect(
		m_hideTimer, &QTimer::timeout, [=]() {
			m_isEdgeHotspot = false;
			updateMargin();
			createEdgeHS();
			m_hideTimer->stop();
		}
	);
}

void DesQAutoHideWindow::initialize( LayerSurface *lSurface ) {

	m_layerSurface = lSurface;

	// Create the dock hotspot
	// Also hide the dock if mouse pointer not in the dock surface
	createDockHS();
}

void DesQAutoHideWindow::createEdgeHS() {

	// Creating edge hotspot will remove dock hotspot
	m_dockHotspot->destroy();
	m_edgeHotspot->createHotspot( ( DesQHotSpot::Position )m_dockPos, m_edgeHeight, m_showDuration );
}

void DesQAutoHideWindow::createDockHS() {

	// Creating dock hotspot will remove edge hotspot
	m_edgeHotspot->destroy();

	m_dockHotspot->createHotspot( ( DesQHotSpot::Position )m_dockPos, m_window->size().height(), 0 ); // No duration needed
	m_hideTimer->start();
}

void DesQAutoHideWindow::updateMargin() {

	if ( m_isEdgeHotspot )
		m_layerSurface->setMargins( QMargins( 0, 0, 0, 0 ) );

	else {
		// Setting the layer bottom margin to hide the layer surface
		if ( m_dockPos == DesQHotSpot::Bottom )
			m_layerSurface->setMargins( QMargins( 0, 0, 0, -m_window->size().height() ) );

		else if ( m_dockPos == DesQHotSpot::Left ) // Not needed
			m_layerSurface->setMargins( QMargins( -m_window->size().width(), 0, 0, 0 ) );
	}

	m_layerSurface->apply();
}

DesQHotSpotPrivate::DesQHotSpotPrivate( QScreen *screen, zwf_shell_manager_v2 *wfShell, DesQHotSpot *ptr ) : q_ptr( ptr ) {

	m_wfOutput = zwf_shell_manager_v2_get_wf_output( wfShell, DesQWayland::getWlOutput( screen ) );
}

void DesQHotSpotPrivate::createHotspot( int pos, uint height, uint duration ) {

	init( zwf_output_v2_create_hotspot( m_wfOutput, pos, height, duration ) );
}

void DesQHotSpotPrivate::destroy() {

	if ( object() ) {
		zwf_hotspot_v2_destroy( object() );
	}
}

void DesQHotSpotPrivate::zwf_hotspot_v2_enter() {

	Q_Q( DesQHotSpot );
	emit q->enteredHotspot();
}

void DesQHotSpotPrivate::zwf_hotspot_v2_leave() {

	Q_Q( DesQHotSpot );
	emit q->leftHotspot();
}

DesQHotSpot::DesQHotSpot( QScreen *screen, zwf_shell_manager_v2 *wfShell, QObject *parent ) : QObject( parent ) {

	d_ptr = new DesQHotSpotPrivate( screen, wfShell, this );
}

void DesQHotSpot::createHotspot( DesQHotSpot::Position pos, uint height, uint duration ) {

	Q_D( DesQHotSpot );
	d->createHotspot( pos, height, duration );
}

void DesQHotSpot::destroy() {

	Q_D( DesQHotSpot );
	d->destroy();
}

void DesQHotSpot::zwf_hotspot_v2_enter() {

	Q_D( DesQHotSpot );
	d->zwf_hotspot_v2_enter();
}

void DesQHotSpot::zwf_hotspot_v2_leave() {

	Q_D( DesQHotSpot );
	d->zwf_hotspot_v2_leave();
}
