/*
	*
	* This file is a part of IonShell.
	* IonShell is the Desktop Shell App for Ion DE
	* Copyright 2020 CuboCore Group
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#include <QDebug>
#include <QWindow>

#include <wayland-client.h>

#include "LayerShell.hpp"
#include "LayerShellImpl.hpp"
#include "DesQWlGlobal.hpp"

#include "qwayland-wlr-layer-shell-unstable-v1.h"

// Easier to type
typedef struct ::zwlr_layer_shell_v1 wlrLayerShell;
typedef struct ::zwlr_layer_surface_v1 wlrLayerSurface;

LayerShellPrivate::LayerShellPrivate( wlrLayerShell *lShell, LayerShell *ptr ) : QtWayland::zwlr_layer_shell_v1( lShell ) {

	q_ptr = ptr;
	qDebug() << "LayerShell constructor";
}

zwlr_layer_surface_v1 *LayerShellPrivate::getLayerSurface( QWindow *window, LayerShell::LayerType lt, const QString &layer_namespace ) {

	if ( !window ) {
		qDebug() << "No window found to create surface";
		return nullptr;
	}

	auto surface = DesQWayland::getWlSurface( window );
	if ( !surface ) {
		qDebug() << "No surface for the given window";
		return nullptr;
	}

	auto output = DesQWayland::getWlOutput( window->screen() );
	if ( !output ) {
		qDebug() << "No output for the given window";
		return nullptr;
	}

	return get_layer_surface( surface, output, static_cast<uint32_t>( lt ), layer_namespace );
}

LayerShell::LayerShell( wlrLayerShell *lShell, QObject *parent ) : QObject( parent ) {

	d_ptr = new LayerShellPrivate( lShell, this );
}

wlrLayerSurface* LayerShell::getLayerSurface( QWindow *window, LayerType layer, const QString &lnamespace ) {

	Q_D( LayerShell );
	return d->getLayerSurface( window, layer, lnamespace );
}

LayerSurfacePrivate::LayerSurfacePrivate( QWindow *window, LayerShell *lShell, LayerShell::LayerType type, LayerSurface *ptr ) :
		QtWayland::zwlr_layer_surface_v1( lShell->getLayerSurface( window, type, "qt" ) ),
		q_ptr( ptr )
	{

	qDebug() << "LayerSurface constructor";

	m_window = window;
	isConfigured = false;

	m_exclusiveZone = 0;
	m_keyboardInteractivity = false;
	m_surfaceSize = QSize();
	m_margins = QMargins( 0, 0, 0, 0 );
};

void LayerSurfacePrivate::apply() {

	isConfigured = false;
	set_anchor( static_cast<uint32_t>( m_anchors ) );
	set_exclusive_zone( m_exclusiveZone );
	set_keyboard_interactivity( m_keyboardInteractivity ? 1 : 0 );

	if ( m_surfaceSize.isValid() )
		set_size( m_surfaceSize.width(), m_surfaceSize.height() );

	set_margin( m_margins.top(), m_margins.right(), m_margins.bottom(), m_margins.left() );

	wl_surface_commit( DesQWayland::getWlSurface( m_window ) );
	wl_display_roundtrip( DesQWayland::getWlDisplay() );
}

void LayerSurfacePrivate::zwlr_layer_surface_v1_configure( uint32_t serial, uint32_t width, uint32_t height ) {

	Q_Q( LayerSurface );

	qDebug() << "Surface configured at " << width << height;

	isConfigured = true;
	m_surfacePendingSize = QSize( width, height );

	ack_configure( serial );
	m_window->resize( m_surfacePendingSize );

	emit q->configured( m_surfacePendingSize );
};

void LayerSurfacePrivate::zwlr_layer_surface_v1_closed() {

	qDebug() << "Closing surface";
	m_window->close();
};

LayerSurface::LayerSurface( QWindow *window, QObject *parent ) : QObject( parent ) {

	mWindow = window;
};

void LayerSurface::setup( LayerShell *lShell, LayerShell::LayerType type ) {

	d_ptr = new LayerSurfacePrivate( mWindow, lShell, type, this );
}

void LayerSurface::apply() {

	Q_D( LayerSurface );
	d->apply();
};

void LayerSurface::setSurfaceSize( const QSize &surfaceSize ) {

	Q_D( LayerSurface );
	d->m_surfaceSize = surfaceSize;
};

void LayerSurface::setAnchors( const SurfaceAnchors &anchors ) {

	Q_D( LayerSurface );
	d->m_anchors = anchors;
};

void LayerSurface::setExclusiveZone( int exclusiveZone ) {

	Q_D( LayerSurface );
	d->m_exclusiveZone = exclusiveZone;
};

void LayerSurface::setMargins( const QMargins &margins ) {

	Q_D( LayerSurface );
	d->m_margins = margins;
};

void LayerSurface::setKeyboardInteractivity( bool keyboardInteractivity ) {

	Q_D( LayerSurface );
	d->m_keyboardInteractivity = keyboardInteractivity;
};
