/*
	*
	* This file is a part of IonShell.
	* IonShell is the Desktop Shell App for Ion DE
	* Copyright 2020 CuboCore Group
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#include <QDebug>

#include "WindowManager.hpp"
#include "WindowManagerImpl.hpp"

WindowManagerPrivate::WindowManagerPrivate(
		struct ::zwlr_foreign_toplevel_manager_v1 *tlManager, struct wl_seat *seat, WindowManager *ptr
	) : QtWayland::zwlr_foreign_toplevel_manager_v1(tlManager), q_ptr( ptr ) {

	m_seat = seat;
	m_tlHandleList.clear();
	qDebug() << "WindowManager constructor";
}

void WindowManagerPrivate::zwlr_foreign_toplevel_manager_v1_toplevel( zwlr_foreign_toplevel_handle_v1 *handle ) {

	WindowHandle *tlHandle = new WindowHandle( handle );
	QObject::connect( tlHandle, &WindowHandle::handleClosed, q_ptr, &WindowManager::closeHandle );
	m_tlHandleList.append( tlHandle );

	Q_Q( WindowManager );
	emit q->newWindowHandle( tlHandle );
}

void WindowManagerPrivate::zwlr_foreign_toplevel_manager_v1_finished() {

	// Does nothing
	qDebug() << "Toplevel Manager finished listening toplevel entries";
}

WindowManager::WindowManager(
		struct ::zwlr_foreign_toplevel_manager_v1 *tlManager, struct wl_seat *seat, QObject *parent
	) : QObject( parent ) {

	d_ptr = new WindowManagerPrivate( tlManager, seat, this );
}

WindowHandles WindowManager::windowHandles() const {

	Q_D( const WindowManager );
	return d->m_tlHandleList;
}

void WindowManager::activateTask( WindowHandle *handle ) {

	Q_D( WindowManager );
	handle->activate( d->m_seat );
}

void WindowManager::closeHandle( WindowHandle *tlHandle ) {

	Q_D( WindowManager );
	d->m_tlHandleList.removeAll( tlHandle );
}

void WindowManager::zwlr_foreign_toplevel_manager_v1_toplevel( zwlr_foreign_toplevel_handle_v1 *handle ) {

	Q_D( WindowManager );
	d->zwlr_foreign_toplevel_manager_v1_toplevel( handle );
}

void WindowManager::zwlr_foreign_toplevel_manager_v1_finished() {
}

WindowHandlePrivate::WindowHandlePrivate(
		struct ::zwlr_foreign_toplevel_handle_v1 *tlHandle, WindowHandle *ptr
	) : QtWayland::zwlr_foreign_toplevel_handle_v1(tlHandle) {

	q_ptr = ptr;

	m_appID = "";
	m_appTitle = "";
	m_appStates = WindowHandle::WindowStates();
}

void WindowHandlePrivate::zwlr_foreign_toplevel_handle_v1_title(const QString &title) {

	m_appTitle = title;
}

void WindowHandlePrivate::zwlr_foreign_toplevel_handle_v1_app_id(const QString &app_id) {

	m_appID = app_id;
}

void WindowHandlePrivate::zwlr_foreign_toplevel_handle_v1_output_enter(wl_output *output) {

	Q_UNUSED(output)
}

void WindowHandlePrivate::zwlr_foreign_toplevel_handle_v1_output_leave(wl_output *output) {

	Q_UNUSED(output)
}

void WindowHandlePrivate::zwlr_foreign_toplevel_handle_v1_state(wl_array *state) {

	auto *states = static_cast<uint32_t *>(state->data);
	int numStates = static_cast<int>(state->size / sizeof(uint32_t));

	for (int i = 0; i < numStates; i++) {
		if (states[i] == ZWLR_FOREIGN_TOPLEVEL_HANDLE_V1_STATE_ACTIVATED) {
			m_appStates |= WindowHandle::Activated;
		} else if (states[i] == ZWLR_FOREIGN_TOPLEVEL_HANDLE_V1_STATE_MAXIMIZED) {
			m_appStates |= WindowHandle::Maximized;
		} else if (states[i] == ZWLR_FOREIGN_TOPLEVEL_HANDLE_V1_STATE_MINIMIZED) {
			m_appStates |= WindowHandle::Minimized;
		}
	}
}

void WindowHandlePrivate::zwlr_foreign_toplevel_handle_v1_done() {

	Q_Q( WindowHandle );
	emit q->handleUpdated();
}

void WindowHandlePrivate::zwlr_foreign_toplevel_handle_v1_closed() {

	Q_Q( WindowHandle );
	emit q->handleClosed( q_ptr );
}


WindowHandle::WindowHandle( struct ::zwlr_foreign_toplevel_handle_v1 *tlHandle, QObject *parent ) : QObject( parent ) {

	d_ptr = new WindowHandlePrivate( tlHandle, this );
}

void WindowHandle::zwlr_foreign_toplevel_handle_v1_title(const QString &title) {

	Q_D( WindowHandle );
	d->zwlr_foreign_toplevel_handle_v1_title( title );
}

void WindowHandle::zwlr_foreign_toplevel_handle_v1_app_id(const QString &app_id) {

	Q_D( WindowHandle );
	d->zwlr_foreign_toplevel_handle_v1_app_id( app_id );
}

void WindowHandle::zwlr_foreign_toplevel_handle_v1_output_enter(wl_output *output) {

	Q_D( WindowHandle );
	d->zwlr_foreign_toplevel_handle_v1_output_enter( output );
}

void WindowHandle::zwlr_foreign_toplevel_handle_v1_output_leave(wl_output *output) {

	Q_D( WindowHandle );
	d->zwlr_foreign_toplevel_handle_v1_output_leave( output );
}

void WindowHandle::zwlr_foreign_toplevel_handle_v1_state(wl_array *states) {

	Q_D( WindowHandle );
	d->zwlr_foreign_toplevel_handle_v1_state( states );
}

void WindowHandle::zwlr_foreign_toplevel_handle_v1_done() {

	Q_D( WindowHandle );
	d->zwlr_foreign_toplevel_handle_v1_done();
}

void WindowHandle::zwlr_foreign_toplevel_handle_v1_closed() {

	Q_D( WindowHandle );
	d->zwlr_foreign_toplevel_handle_v1_closed();
}

QString WindowHandle::appID() const {

	Q_D( const WindowHandle );
	return d->m_appID;
}

QString WindowHandle::appTitle() const {

	Q_D( const WindowHandle );
	return d->m_appTitle;
}

void WindowHandle::set_maximized() {

	Q_D( WindowHandle );
	d->unset_maximized();
}

void WindowHandle::unset_maximized() {

	Q_D( WindowHandle );
	d->set_maximized();
}

void WindowHandle::set_minimized() {

	Q_D( WindowHandle );
	d->unset_minimized();
}

void WindowHandle::unset_minimized() {

	Q_D( WindowHandle );
	d->set_minimized();
}

void WindowHandle::activate( struct ::wl_seat *seat ) {

	Q_D( WindowHandle );
	d->activate( seat );
}

void WindowHandle::close() {

	Q_D( WindowHandle );
	d->close();
}


WindowHandle::WindowStates WindowHandle::appStates() const {

	Q_D( const WindowHandle );
	return d->m_appStates;
}
